/*
 * ntctemp.c
 *
 * Created: 2015-06-12 14:12:08
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000UL
#include <avr/io.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "defs.h"
#include "ntctemp.h"
#include "comm.h"

//--------------------------------------------
#define POT_TABLE_LEN		50
#define NTC_TABLE_LEN		150

const char delta_pot[POT_TABLE_LEN + 1];
const char delta_ntc[NTC_TABLE_LEN + 1];

#define ADC_CONFIG1			(0<<REFS2 | 0<<REFS1 | 0<<ADLAR)	//vcc ref
#define ADC_CONFIG2			(1<<ADEN | 1<<ADPS2 | 1<<ADPS0)		//presc 32
#define ADC_START			(1<<ADSC)


//--------------------------------------------

struct {
	char itemp;
	char ivolume;
	int16_t temperature;
	char status;
	int16_t volume;
} ntc;

struct {
	uint16_t aku[ANALOG_CHANNELS_NUM];
	uint16_t counter;
} analog;


//----------------------------------------
void NtcInit(void)
{
	DIDR0 = (1<<ADC2D | 1<<ADC3D);
}

//----------------------------------------
char AnalogProc(void)
{
	static uint16_t i;
	
	if (analog.counter < ANALOG_AVERAGE_COUNT)
	{
		PRR &= ~(1<<PRADC);
		ADMUX = ADC_CONFIG1;
		ADCSRA = ADC_CONFIG2;
		ADCSRB = 0;
	
		i = AnalogAdc(ANALOG_NTC_CHANNEL);
		analog.aku[0] += AnalogAdc(ANALOG_NTC_CHANNEL);
		analog.aku[1] += AnalogAdc(ANALOG_VOLUME_CHANNEL);

		ADCSRA &= ~(1<<ADEN);
		PRR = POWER_CONFIG;

		analog.counter++;
	}

	if (analog.counter >= ANALOG_AVERAGE_COUNT)
	{
		return 1;
	}
	return 0;
}

//---------------------------------
uint16_t AnalogAdc(char channel)
{
	while (ADCSRA & (1<<ADSC));			//wait for conv complite
	ADMUX = (ADC_CONFIG1 | channel);	//select channel
	ADCSRA = (ADC_CONFIG2 | ADC_START);	//on and start
	while (ADCSRA & (1<<ADSC));			//wait for conv complite
	return ADC;						//return val
}

//---------------------------------
uint16_t AnalogGet(char channel)
{
	if (channel == ANALOG_NTC_CHANNEL)
	{
		return (analog.aku[0] / analog.counter);
	}
	if (channel == ANALOG_VOLUME_CHANNEL)
	{
		return (analog.aku[1] / analog.counter);
	}
	return 0;
}

//---------------------------------
void AnalogRestart(void)
{
	analog.aku[0] = 0;
	analog.aku[1] = 0;
	analog.counter = 0;
}

//----------------------------------------
int16_t NtcTempCalc(uint16_t adcval)
{
	if ((adcval <= 1) || (adcval >= 1020))
		ntc.status |=  NTC_SENSOR_ERROR;
	else
		ntc.status &= ~NTC_SENSOR_ERROR;

	ntc.itemp = NtcTempIndex(adcval);
	ntc.temperature = (int16_t)(NTC_RANGE_MIN + ntc.itemp);
	return ntc.temperature;
}

//----------------------------------------
int16_t NtcVolumeCalc(uint16_t adcval)
{
	if (adcval >= 1020)
		ntc.status |=  NTC_VOLUME_ERROR;
	else
		ntc.status &= ~NTC_VOLUME_ERROR;

	ntc.ivolume = NtcVolumeIndex(adcval);
	//ntc.volume = (int16_t)(ntc.offset_volume + ntc.ivolume);
	ntc.volume = (int16_t)(SettRangeGet() + ntc.ivolume);
	return ntc.volume;
}

//----------------------------------------
int16_t NtcTempGet(void)
{
	return ntc.temperature;
}

//----------------------------------------
int16_t NtcVolumeGet(void)
{
	return ntc.volume;
}

//----------------------------------------
char NtcStatusGet(void)
{
	return ntc.status;
}

//----------------------------------------
char NtcTempIndex(int16_t val)
{
	char enough;
	int16_t i;
	int16_t add;
	
	add = 0;
	i = 0;
	enough = 0;
	while (enough == 0)
	{
		add += (pgm_read_byte(&delta_ntc[i]));
		
		if (add >= val)
		{
			enough = 1;	//ok
		}
		else
		{
			i++;
			if (i > NTC_TABLE_LEN)
			{
				i--;		//last val - min val
				enough = 1;	// out of range
			}
		}
	}
	i = (NTC_TABLE_LEN - i);
	return i;
}

//----------------------------------------
//return 0...50 - min...max reg
char NtcVolumeIndex(int16_t val)
{
	char enough;
	int16_t i;
	int16_t add;
	
	add = 0;
	i = 0;
	enough = 0;
	while (enough == 0)
	{
		add += (pgm_read_byte(&delta_pot[i]));
		
		if (add >= val)
		{
			enough = 1;	//ok
		}
		else
		{
			i++;
			if (i > POT_TABLE_LEN)
			{
				i--;		//last val - min val
				enough = 1;	// out of range
			}
		}
	}
	i = (POT_TABLE_LEN - i);
	return i;
}


//R=3k;  U:0...3,85;  ADC:0...788; 0=max pot volume (0R)
const char delta_pot[POT_TABLE_LEN + 1]  PROGMEM = {
	0,
	64,
	56,
	50,
	45,
	40,
	37,
	33,
	30,
	28,
	26,
	24,
	22,
	20,
	19,
	18,
	17,
	15,
	15,
	14,
	13,
	12,
	12,
	11,
	10,
	10,
	9,
	9,
	9,
	8,
	8,
	7,
	7,
	7,
	7,
	6,
	6,
	6,
	6,
	5,
	5,
	5,
	5,
	5,
	4,
	4,
	4,
	4,
	4,
	4,
	4
};


//R=3k; NTC=10k; T=-25C...125C; ADC:104...1001, 0-max temp
const char delta_ntc[NTC_TABLE_LEN + 1]  PROGMEM = {
	104,
	2,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	4,
	4,
	4,
	4,
	4,
	4,
	4,
	4,
	4,
	4,
	5,
	5,
	5,
	5,
	5,
	5,
	5,
	5,
	5,
	6,
	6,
	6,
	6,
	6,
	6,
	6,
	6,
	7,
	7,
	7,
	7,
	7,
	7,
	7,
	8,
	7,
	8,
	8,
	8,
	8,
	8,
	8,
	8,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	10,
	10,
	10,
	10,
	10,
	9,
	10,
	10,
	10,
	10,
	10,
	10,
	10,
	10,
	10,
	10,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	9,
	8,
	8,
	8,
	8,
	8,
	8,
	8,
	7,
	7,
	7,
	7,
	7,
	7,
	6,
	6,
	6,
	6,
	6,
	6,
	5,
	5,
	5,
	5,
	5,
	5,
	4,
	4,
	4,
	4,
	4,
	4,
	4,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	3,
	2,
	2,
	2,
	2,
	2,
	2,
	2,
	2,
	2,
	2,
	2,
	1,
	1
};